/**
 * This small script let user easily switch from c (or cpp) files to h (or hpp)
 * - File is open in another tab if it's not already open. 
 */

/**
 * CUSTOM: 
 *  - you can change text in log area (search logAddLine function).
 *  - this can be extended to other language by changing C_Extension and H_Extension array.  
 *  - change your short-cut (search addMenuItem).
 *  TODO:
 *  - maybe option to create file if not exists yet.     
 */

/**
 * History
 * who when       what 
 * pp  09/02/2007 0.01 initial release
 * pp  21/12/2007 0.02 upgrade due to log evolution  logAddLine() replace reportLog.appendText
 */
var module_name = "CodeHeaderJS";
var module_ver = "0.02";

var C_Extension = ["c","cpp"];// use to switch from (h or hpp) to c or cpp
var H_Extension = ["h","hpp"];
var undefinedValue;// must stay undefined for undefined test
var reportLog;

/**
 */
function xtractFile_withExt(data){
    var m = data.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/);
    if ((m === null) || (m.length === undefinedValue))
        throw new Error(" cannot parse :"+data);
    return {path: m[1], file: m[2]};
};

/**
 */
function xtractFile(data){
    data = data.replace(/^\s|\s$/g, ""); //trims string

    if (/\.\w+$/.test(data)) {
        if (data.match(/([^\/\\]+)\.(\w+)$/) )
            return {filename: RegExp.$1, ext: RegExp.$2};
        else
            return {filename: "no file name", ext:null};
    }
    else {
        if (data.match(/([^\/\\]+)$/) )
            return {filename: RegExp.$1, ext: null};
        else
            return {filename: "no file name", ext:null};
    }
}

/**
 */
function tryOpenFile(file, extarray){
    var b = false;
    var i = 0;
    var expectedFile;
    while((!b)&&(i<extarray.length))
    {
        var ext = extarray[i];
        expectedFile = file + ext;

        try
        {
            assignEditorByName(expectedFile);
            b = true;
        }
        catch(e)
        {
            try
            {
                var fs = new ActiveXObject("Scripting.FileSystemObject");
                if (fs.fileExists(expectedFile))
                {
                    var ne = newEditor();
                    ne.openFile(expectedFile);
                    ne = null;
                    b = true;
                }
            }
            catch(e)
            {
            }
        }
        i++;
    }
    if (!b)
        throw new Error(" with extension(s) :["+extarray.join()+"]");
};

/**
 */
function knownExtension(ext, extarray){
    var b = false;
    var i = 0;
    
    while((!b)&&(i<extarray.length))
    {
        b = (String(extarray[i]).toUpperCase() === String(ext).toUpperCase());
        i++;
    }
    return b;
};

/**
 */
function switchcodeheader() {
    var ed  = newEditor();
    ed.assignActiveEditor();

    //reportLog = newEditor();
    //reportLog.assignLog();
    //reportLog.Text("");   
    var fn = ed.fileName();
    // extract path, file, extension
    try
    {
        var fp = xtractFile_withExt(fn);
        var fPath = fp.path;
        var fe = xtractFile(fp.file);
        if (fe.ext != null)
        {
            var expectedFile;
            if (knownExtension(fe.ext, C_Extension))
            {
                tryOpenFile(fPath + "\\" + fe.filename + ".", H_Extension);
            }
            else
            if (knownExtension(fe.ext, H_Extension))
            {
                tryOpenFile(fPath + "\\" + fe.filename + ".", C_Extension);
            }
            else
            {
                throw new Error(" unmanaged extension");
            }
        }
    }
    catch(e)
    {
        logAddLine("Find nothing for file :"+fn+"\r\n"+e.message);
        //reportLog.appendText("Find nothing for file :"+fn+"\r\n"+e.message);
    }
    fe = null;
    fn = null;
    ed = null;
};

/**
 */
function Init(){
    addMenuItem("Switch from code to header", "C++", "switchcodeheader", "Ctrl+I");
    addMenuItem("-","C++","","");                      // Menu divider
    addMenuItem("Edit this script","C++","editMe");    // Edit this script in PSPad
    addMenuItem("About","C++","about","");             // About this extension
};

/**
 */
function editMe() 
{
    var ne = newEditor();
	ne.openFile(moduleFileName(module_name));
}


/**
 */
function about() 
{
    echo(
        "\n" + module_name + " " + module_ver + "\n\n" +
        "_________________\n" +
        "DESCRIPTION\n\n" +
        "   PSPad Editor extension to swap from source code\n" +
        "   to header file (or by extension to related file).\n"
    );
    return;
}

